/*
 * Copyright (C) The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ecp.vcs.com.ecpsyncplugin.barcode.barcodereader;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.content.res.AssetFileDescriptor;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.hardware.Camera;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.media.ToneGenerator;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;

import android.util.Log;
import android.view.GestureDetector;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.ScaleGestureDetector;
import android.view.View;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatButton;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.core.app.ActivityCompat;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;
import com.google.android.gms.common.api.CommonStatusCodes;
import com.google.android.gms.vision.MultiProcessor;
import com.google.android.gms.vision.barcode.Barcode;
import com.google.android.gms.vision.barcode.BarcodeDetector;
import com.google.android.material.snackbar.Snackbar;

import org.json.JSONArray;

import java.io.IOException;
import java.util.ArrayList;

import ecp.vcs.com.ecpsyncplugin.R;
import ecp.vcs.com.ecpsyncplugin.Utils;
import ecp.vcs.com.ecpsyncplugin.barcode.barcodereader.ui.camera.CameraSource;
import ecp.vcs.com.ecpsyncplugin.barcode.barcodereader.ui.camera.CameraSourcePreview;
import ecp.vcs.com.ecpsyncplugin.barcode.barcodereader.ui.camera.GraphicOverlay;

/**
 * Activity for the multi-tracker app.  This app detects barcodes and displays the value with the
 * rear facing camera. During detection overlay graphics are drawn to indicate the position,
 * size, and ID of each barcode.
 */
public final class BarcodeCaptureActivity extends Activity implements BarcodeGraphicTracker.BarcodeDetectorListener, View.OnClickListener {
    private Context context = BarcodeCaptureActivity.this;
    private static final String TAG = "Barcode-reader";
    // intent request code to handle updating play service if needed.
    private static final int RC_HANDLE_GMS = 9001;
    // permission request codes need to be < 256
    private static final int RC_HANDLE_CAMERA_PERM = 2;
    // constants used to pass extra data in the intent
    public static final String AutoFocus = "AutoFocus";
    //    public static final String UseFlash = "UseFlash";
    public static boolean UseFlash = false;
    public static final String BarcodeObject = "Barcode";
    public static final String LIST_BARCODES = "List_Barcode";

    private CameraSource mCameraSource;
    private CameraSourcePreview mPreview;
    private GraphicOverlay<BarcodeGraphic> mGraphicOverlay;

    // helper objects for detecting taps and pinches.
    private ScaleGestureDetector scaleGestureDetector;
    private GestureDetector gestureDetector;

    public static String detectedBarCode = "";
    //private RecyclerView recyclerView;
    private ArrayList<String> barCodeList;
    private TextView tvBarCodeCount;
    private Button bnDone;
    CameraSource.Builder builder;
    //ScannerOverlay scannerOverlayView;
    //private AppPrefs mAppPrefs;
//    private Animation animBounce;
    String isSingleScan;
    String isPlaySound;
    String isOnFlase;

    /**
     * Initializes the UI and creates the detector pipeline.
     */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);

        isSingleScan = getIntent().getExtras().getString("isSingleScan", "false");
        isPlaySound = getIntent().getExtras().getString("isPlaySound", "false");
        isOnFlase = getIntent().getExtras().getString("isOnFlase", "false");
        String themeColor = getIntent().getExtras().getString("themeColor", "false");
        String nextBtn = getIntent().getExtras().getString("nextBtn", "Next");
        String scanText = getIntent().getExtras().getString("scanText", "Scan your 2D label here...");

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            getWindow().setStatusBarColor(Color.parseColor(themeColor.equalsIgnoreCase("") ? "#004a96" : themeColor));
        }
        setContentView(R.layout.barcode_capture);

        assert getActionBar() != null;
        getActionBar().setTitle("");
        getActionBar().setDisplayHomeAsUpEnabled(true);
        //getActionBar().setHomeAsUpIndicator(R.drawable.back_dark);
        getActionBar().setBackgroundDrawable(new ColorDrawable(Color.parseColor(themeColor.equalsIgnoreCase("") ? "#004a96" : themeColor)));

        //mAppPrefs = new AppPrefs(this);

        barCodeList = new ArrayList<>();
        TextView labScanText = findViewById(R.id.labScanText);
        labScanText.setText(scanText);
        mPreview = (CameraSourcePreview) findViewById(R.id.preview);

        //scannerOverlayView = findViewById(R.id.scannerOverlayView);
        //scannerOverlayView.shouldDelayChildPressedState();

        mGraphicOverlay = (GraphicOverlay<BarcodeGraphic>) findViewById(R.id.graphicOverlay);

        //mGraphicOverlay.clear();

        tvBarCodeCount = findViewById(R.id.tvBarCodeScanCount);
        bnDone = findViewById(R.id.bnDone);
        bnDone.setText(nextBtn);
        bnDone.setOnClickListener(this);

        /*recyclerView = (RecyclerView) findViewById(R.id.recycler_view);
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(getApplicationContext());
        recyclerView.setLayoutManager(mLayoutManager);
        recyclerView.setItemAnimator(new DefaultItemAnimator());*/

        // read parameters from the intent used to launch the activity.
        boolean autoFocus = getIntent().getBooleanExtra(AutoFocus, false);
        //boolean useFlash = getIntent().getBooleanExtra(UseFlash, false);

        BarcodeGraphicTracker.mBarcodeDetectorListener = this;

        // Check for the camera permission before accessing the camera.  If the
        // permission is not granted yet, request permission.
        int rc = ActivityCompat.checkSelfPermission(this, Manifest.permission.CAMERA);
        if (rc == PackageManager.PERMISSION_GRANTED) {
            createCameraSource(autoFocus, UseFlash);
        } else {
            requestCameraPermission();
        }

        gestureDetector = new GestureDetector(this, new CaptureGestureListener());
        scaleGestureDetector = new ScaleGestureDetector(this, new ScaleListener());

//        Snackbar.make(mGraphicOverlay, "Tap to capture. Pinch/Stretch to zoom",
//                Snackbar.LENGTH_LONG)
//                .show();
    }

    /**
     * Handles the requesting of the camera permission.  This includes
     * showing a "Snackbar" message of why the permission is needed then
     * sending the request.
     */
    private void requestCameraPermission() {
        Log.w(TAG, "Camera permission is not granted. Requesting permission");

        final String[] permissions = new String[]{Manifest.permission.CAMERA};

        if (!ActivityCompat.shouldShowRequestPermissionRationale(this,
                Manifest.permission.CAMERA)) {
            ActivityCompat.requestPermissions(this, permissions, RC_HANDLE_CAMERA_PERM);
            return;
        }

        final Activity thisActivity = this;

        View.OnClickListener listener = new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                ActivityCompat.requestPermissions(thisActivity, permissions,
                        RC_HANDLE_CAMERA_PERM);
            }
        };

        findViewById(R.id.topLayout).setOnClickListener(listener);
        Snackbar.make(mGraphicOverlay, "Access to the camera is needed for detection",
                Snackbar.LENGTH_INDEFINITE)
                .setAction("OK", listener)
                .show();
    }

    @Override
    public boolean onTouchEvent(MotionEvent e) {
        boolean b = scaleGestureDetector.onTouchEvent(e);

        boolean c = gestureDetector.onTouchEvent(e);

        return b || c || super.onTouchEvent(e);
    }

    /**
     * Creates and starts the camera.  Note that this uses a higher resolution in comparison
     * to other detection examples to enable the barcode detector to detect small barcodes
     * at long distances.
     * <p/>
     * Suppressing InlinedApi since there is a check that the minimum version is met before using
     * the constant.
     */
    @SuppressLint("InlinedApi")
    private void createCameraSource(boolean autoFocus, boolean useFlash) {
        Context context = getApplicationContext();
        //mGraphicOverlay.clear();
        BarcodeDetector barcodeDetector = new BarcodeDetector.Builder(context).build();
        BarcodeTrackerFactory barcodeFactory = new BarcodeTrackerFactory(mGraphicOverlay);
        barcodeDetector.setProcessor(new MultiProcessor.Builder<>(barcodeFactory).build());

        if (!barcodeDetector.isOperational()) {
            Log.e(TAG, "Detector dependencies are not yet available.");
            IntentFilter lowstorageFilter = new IntentFilter(Intent.ACTION_DEVICE_STORAGE_LOW);
            boolean hasLowStorage = registerReceiver(null, lowstorageFilter) != null;
            if (hasLowStorage) {
                Log.e(TAG, "Face detector dependencies cannot be downloaded due to low device storage");
                Toast.makeText(BarcodeCaptureActivity.this, "Face detector dependencies cannot be downloaded due to low device storage", Toast.LENGTH_SHORT).show();
            }
        }

        builder = new CameraSource.Builder(getApplicationContext(), barcodeDetector)
                .setFacing(CameraSource.CAMERA_FACING_BACK)
                .setRequestedPreviewSize(1600, 1024)
                .setRequestedFps(15.0f);
        // make sure that auto focus is an available option
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
            builder = builder.setFocusMode(
                    autoFocus ? Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE : null);
        }

        mCameraSource = builder
                .setFlashMode(useFlash ? Camera.Parameters.FLASH_MODE_TORCH : null)
                .build();
    }

    /**
     * Restarts the camera.
     */
    @Override
    protected void onResume() {
        super.onResume();
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        UseFlash = false;
        //setUpFlashView(tvFlash);
        startCameraSource();
    }

    /**
     * Stops the camera.
     */
    @Override
    protected void onPause() {
        super.onPause();
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        if (mPreview != null) {
            mPreview.stop();
        }

    }

    /**
     * Releases the resources associated with the camera source, the associated detectors, and the
     * rest of the processing pipeline.
     */
    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mPreview != null) {
            mPreview.release();
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        if (requestCode != RC_HANDLE_CAMERA_PERM) {
            Log.d(TAG, "Got unexpected permission result: " + requestCode);
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
            return;
        }

        if (grantResults.length != 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            Log.d(TAG, "Camera permission granted - initialize the camera source");
            // we have permission, so create the camerasource
            boolean autoFocus = getIntent().getBooleanExtra(AutoFocus, false);
            //boolean useFlash = getIntent().getBooleanExtra(UseFlash, false);
            createCameraSource(autoFocus, UseFlash);
            return;
        }

        Log.e(TAG, "Permission not granted: results len = " + grantResults.length +
                " Result code = " + (grantResults.length > 0 ? grantResults[0] : "(empty)"));
        // user rejected the permission
        boolean showRationale = shouldShowRequestPermissionRationale("android.permission.CAMERA");
        if (!showRationale) {
            /*Utility.showDialogMessage(context, getResources().getString(R.string.title_common), getResources().getString(R.string.permission_camera_rationale), getResources().getString(R.string.ok), false, new DialogListener() {
                @Override
                public void onConfirm() {
                    Utility.openAppSettings(context);
                }

                @Override
                public void onCancel() {

                }
            });*/
            Toast.makeText(context, "Access to the camera is needed for detection", Toast.LENGTH_SHORT).show();

        } else {
            ActivityCompat.requestPermissions(this, permissions,
                    RC_HANDLE_CAMERA_PERM);
        }


    }

    /**
     * Starts or restarts the camera source, if it exists.  If the camera source doesn't exist yet
     * (e.g., because onResume was called before the camera source was created), this will be called
     * again when the camera source is created.
     */
    private void startCameraSource() throws SecurityException {
        // check that the device has play service available.
        int code = GoogleApiAvailability.getInstance().isGooglePlayServicesAvailable(
                getApplicationContext());
        if (code != ConnectionResult.SUCCESS) {
            Dialog dlg =
                    GoogleApiAvailability.getInstance().getErrorDialog(this, code, RC_HANDLE_GMS);
            dlg.show();
        }

        if (mCameraSource != null) {
            try {
                //mPreview.start(mCameraSource, null);
                mPreview.start(mCameraSource, mGraphicOverlay);
            } catch (IOException e) {
                Log.e(TAG, "Unable to start camera source.", e);
                mCameraSource.release();
                mCameraSource = null;
            }
        }
    }

    /*private void setUpFlashView(ImageView flashView) {
        try {
            if (getPackageManager().hasSystemFeature(
                    PackageManager.FEATURE_CAMERA_FLASH)) {
                flashView.setVisibility(View.VISIBLE);
                if (!UseFlash) {
                    flashView.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.ic_flash_on_black_24dp));
                    UseFlash = true;
                    mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_ON);
                    mCameraSource.release();
                    createCameraSource(false, UseFlash);
                    startCameraSource();
                } else {
                    flashView.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.ic_flash_off_black_24dp));
                    UseFlash = false;
                    mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
                    mCameraSource.release();
                    createCameraSource(false, UseFlash);
                    startCameraSource();
                }
            } else {
                flashView.setVisibility(View.GONE);
                try {
                    UseFlash = false;
                    mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
                    mCameraSource.release();
                    createCameraSource(false, UseFlash);
                    startCameraSource();
                } catch (Exception e1) {
                    e1.printStackTrace();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            flashView.setVisibility(View.GONE);
            try {
                UseFlash = false;
                mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
                mCameraSource.release();
                createCameraSource(false, UseFlash);
                startCameraSource();
            } catch (Exception e1) {
                e1.printStackTrace();
            }
        }
    }*/

    /**
     * onTap returns the tapped barcode result to the calling Activity.
     *
     * @param rawX - the raw position of the tap
     * @param rawY - the raw position of the tap.
     * @return true if the activity is ending.
     */
    private boolean onTap(float rawX, float rawY) {
        // Find tap point in preview frame coordinates.
        int[] location = new int[2];
        mGraphicOverlay.getLocationOnScreen(location);
        float x = (rawX - location[0]) / mGraphicOverlay.getWidthScaleFactor();
        float y = (rawY - location[1]) / mGraphicOverlay.getHeightScaleFactor();

        // Find the barcode whose center is closest to the tapped point.
        Barcode best = null;
        float bestDistance = Float.MAX_VALUE;
        for (BarcodeGraphic graphic : mGraphicOverlay.getGraphics()) {
            Barcode barcode = graphic.getBarcode();
            if (barcode.getBoundingBox().contains((int) x, (int) y)) {
                // Exact hit, no need to keep looking.
                best = barcode;
                break;
            }
            float dx = x - barcode.getBoundingBox().centerX();
            float dy = y - barcode.getBoundingBox().centerY();
            float distance = (dx * dx) + (dy * dy);  // actually squared distance
            if (distance < bestDistance) {
                best = barcode;
                bestDistance = distance;
            }
        }

        if (best != null) {
            Intent data = new Intent();
            data.putExtra(BarcodeObject, best);
            setResult(CommonStatusCodes.SUCCESS, data);
            //finish();
            return true;
        }
        return false;
    }


    @Override
    public void onObjectDetected(final Barcode data) {
        // TODO: 2017/07/31 Have a list of identified objects and keep adding them till the user decides it's enough
        //Log.v("SCANNED BARCODE", data.displayValue);
        //beepDevice();
        //String finalString = getFinalString(data.displayValue);
        String finalString = Utils.getSerialNoFromBarcode(data.displayValue, false);
        if (finalString != null && finalString.toString().trim().length() > 0) {
            detectedBarCode = finalString;
            if (isExistBarCode(finalString)) {
                barCodeList.add(finalString);
            }

            try {
                tvBarCodeCount.setText(String.valueOf(barCodeList.size()));
                //tvBarCodeCount.startAnimation(animBounce);
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (Boolean.parseBoolean(isSingleScan)) {
                if (barCodeList.size() > 0) {
                    doneCall();
                }
            }

            if (Boolean.parseBoolean(isPlaySound)) {
                try {
                    /*Uri notification = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
                    Ringtone r = RingtoneManager.getRingtone(getApplicationContext(), notification);
                    r.play();*/

                    AssetFileDescriptor afd = getAssets().openFd("beep_sound.mp3");
                    MediaPlayer player = new MediaPlayer();
                    player.setDataSource(afd.getFileDescriptor(), afd.getStartOffset(), afd.getLength());
                    player.prepare();
                    player.start();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            //beepSound();

        }
    }

    private boolean isExistBarCode(String barcodeValue) {
        if (barcodeValue != null) {
            if (barCodeList.size() > 0) {
                for (String barcode : barCodeList) {
                    if (barcode.equals(barcodeValue)) {
                        return false;
                    }
                }
            }
        }
        return true;
    }

    private String getFinalString(String scanResult) {
        String varString = scanResult.toString();
        varString = varString.replace("\"", "");
        varString = varString.trim();
        int len = varString.length();
        String finalStr = "";
        for (int i = 0; i < len; i++) {
            char k = varString.charAt(i);
            int ascii = (int) k;
            if (ascii != 29) {
                finalStr = finalStr + k;
            } else {
                Log.v("TAG", "29");
            }
        }
        return finalStr;
    }

    private void bindData() {
      /*  runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (listBarcodes.size() > 0) {
                    beepDevice();
                    adpBarcodes adapter = (adpBarcodes) recyclerView.getAdapter();
                    if (adapter != null) {
                        adapter.refreshList(listBarcodes);
                    } else {
                        adapter = new adpBarcodes(listBarcodes);
                        recyclerView.setAdapter(adapter);
                    }
                }
            }
        });*/
    }

    /**
     * Beeps Device when A Barcode is Scanned
     */
    private void beepDevice() {
//        ToneGenerator toneGen1 = new ToneGenerator(AudioManager.STREAM_MUSIC, 50);
//        toneGen1.startTone(ToneGenerator.TONE_PROP_BEEP);

        ToneGenerator toneG = new ToneGenerator(AudioManager.STREAM_ALARM, 100);
        toneG.startTone(ToneGenerator.TONE_PROP_BEEP, 200);

//        ToneGenerator toneG = new ToneGenerator(AudioManager.STREAM_ALARM, 100);
//        toneG.startTone(ToneGenerator.TONE_CDMA_ALERT_CALL_GUARD, 200);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            Intent intent = new Intent("ScanResult");
            intent.putExtra("SCAN", "DONE");
            intent.putExtra("JSONDATA", "0");
            sendBroadcast(intent);
            finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    @Override
    public void onClick(View v) {
        if (v.getId() == R.id.bnDone) {
            /*Intent returnIntent = new Intent();
            returnIntent.putExtra("barcode", barCodeList);
            setResult(Activity.RESULT_OK, returnIntent);
            finish();*/
            doneCall();
        }
    }

    void doneCall() {
        try {
            Log.e("btnNext", "onClick");
            Intent intent = new Intent("ScanResult");
            if (barCodeList.size() > 0) {
                JSONArray mJSONArray = new JSONArray(barCodeList);
                intent.putExtra("JSONDATA", "" + mJSONArray.toString());
            } else {
                intent.putExtra("JSONDATA", "");
            }
            intent.putExtra("SCAN", "DONE");
            sendBroadcast(intent);
            finish();
        } catch (Exception e) {
            e.printStackTrace();
            Intent intent = new Intent("ScanResult");
            intent.putExtra("SCAN", "ERROR");
            intent.putExtra("JSONDATA", "100");
            sendBroadcast(intent);
            finish();
        }
    }

    private class CaptureGestureListener extends GestureDetector.SimpleOnGestureListener {
        @Override
        public boolean onSingleTapConfirmed(MotionEvent e) {
            return onTap(e.getRawX(), e.getRawY()) || super.onSingleTapConfirmed(e);
        }
    }

    private class ScaleListener implements ScaleGestureDetector.OnScaleGestureListener {

        /**
         * Responds to scaling events for a gesture in progress.
         * Reported by pointer motion.
         *
         * @param detector The detector reporting the event - use this to
         *                 retrieve extended info about event state.
         * @return Whether or not the detector should consider this event
         * as handled. If an event was not handled, the detector
         * will continue to accumulate movement until an event is
         * handled. This can be useful if an application, for example,
         * only wants to update scaling factors if the change is
         * greater than 0.01.
         */
        @Override
        public boolean onScale(ScaleGestureDetector detector) {
            return false;
        }

        /**
         * Responds to the beginning of a scaling gesture. Reported by
         * new pointers going down.
         *
         * @param detector The detector reporting the event - use this to
         *                 retrieve extended info about event state.
         * @return Whether or not the detector should continue recognizing
         * this gesture. For example, if a gesture is beginning
         * with a focal point outside of a region where it makes
         * sense, onScaleBegin() may return false to ignore the
         * rest of the gesture.
         */
        @Override
        public boolean onScaleBegin(ScaleGestureDetector detector) {
            return true;
        }

        /**
         * Responds to the end of a scale gesture. Reported by existing
         * pointers going up.
         * <p/>
         * Once a scale has ended, {@link ScaleGestureDetector#getFocusX()}
         * and {@link ScaleGestureDetector#getFocusY()} will return focal point
         * of the pointers remaining on the screen.
         *
         * @param detector The detector reporting the event - use this to
         *                 retrieve extended info about event state.
         */
        @Override
        public void onScaleEnd(ScaleGestureDetector detector) {
            mCameraSource.doZoom(detector.getScaleFactor());
        }
    }
}
