import 'dart:async';

import 'package:flutter/services.dart';

enum BatteryState { full, charging, discharging }

class EcpSyncPlugin {
  static const MethodChannel _channel = const MethodChannel('ecp_sync_plugin');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static EcpSyncPlugin _instance;

  factory EcpSyncPlugin() {
    if (_instance == null) {
      final MethodChannel methodChannel =
          const MethodChannel('ecp_sync_plugin');
      final EventChannel eventChannel =
          const EventChannel('ecp_sync_plugin_Progress');
      _instance = EcpSyncPlugin.private(methodChannel, eventChannel);
    }
    return _instance;
  }

  final MethodChannel _methodChannel;
  final EventChannel _eventChannel;

  Stream<Map> _onBatteryStateChanged;

  EcpSyncPlugin.private(this._methodChannel, this._eventChannel);

  Future<String> syncronizeData(
      {String userName,
      String fkEmpGlCode,
      String urlUpload,
      String iMei,
      String appVersion,
      String uploadMethod,
      String clientName,
      String databaseName,
      String packageName,
      String dbPassword,
      String isDbEncripted,
      String methodSyncID}) async {
    /*final Map<String, dynamic> params = <String, dynamic>{
      'UserName': UserName,
      'fk_EmpGlCode': fk_EmpGlCode,
      'ClientName': ClientName,
      'imei': imei,
      'version': appVersion,
      'uploadMethod': uploadMethod,
      'urlUpload': urlUpload,
      'databaseName': databaseName,
      'packageName': packageName3
    };*/

    //String jsonData = JsonEncoder(params).toString();

    final String version = await _methodChannel.invokeMethod('syncronizeData', {
      'UserName': userName,
      'fk_EmpGlCode': fkEmpGlCode,
      'ClientName': clientName,
      'imei': iMei,
      'version': appVersion,
      'uploadMethod': uploadMethod,
      'urlUpload': urlUpload,
      'databaseName': databaseName,
      'packageName': packageName,
      'dbPassword': dbPassword,
      'isDbEncripted': isDbEncripted,
      'methodSyncID': methodSyncID
    });

    return version;
  }

  Future<String> syncronizeDataFile(
      {String userName,
      String fkEmpGlCode,
      String urlUpload,
      String iMei,
      String appVersion,
      String uploadMethod,
      String logUploadMethod,
      String clientName,
      String databaseName,
      String packageName,
      String dbPassword,
      String isDbEncripted,
      String methodSyncID}) async {
    /*final Map<String, dynamic> params = <String, dynamic>{
      'UserName': UserName,
      'fk_EmpGlCode': fk_EmpGlCode,
      'ClientName': ClientName,
      'imei': imei,
      'version': appVersion,
      'uploadMethod': uploadMethod,
      'urlUpload': urlUpload,
      'databaseName': databaseName,
      'packageName': packageName3
    };*/

    //String jsonData = JsonEncoder(params).toString();

    final String version =
        await _methodChannel.invokeMethod('syncronizeDataFile', {
      'UserName': userName,
      'fk_EmpGlCode': fkEmpGlCode,
      'ClientName': clientName,
      'imei': iMei,
      'version': appVersion,
      'uploadMethod': uploadMethod,
      'logUploadMethod': logUploadMethod,
      'urlUpload': urlUpload,
      'databaseName': databaseName,
      'packageName': packageName,
      'dbPassword': dbPassword,
      'isDbEncripted': isDbEncripted,
      'methodSyncID': methodSyncID
    });

    return version;
  }

  Future<String> encodeData(String details, String key, String iv) async {
    final String version = await _methodChannel
        .invokeMethod('encodeData', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> decodeData(String details, String key, String iv) async {
    final String version = await _methodChannel
        .invokeMethod('decodeData', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> reEncryption(String details, String key, String iv) async {
    final String version = await _methodChannel
        .invokeMethod('RE_Encryption', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> reDescription(String details, String key, String iv) async {
    final String version = await _methodChannel
        .invokeMethod('RE_Description', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> hideKeyboardPlugin() async {
    final String version =
        await _methodChannel.invokeMethod('hideKeyboardPlugin');

    return version;
  }

  /*Future<String> decodeData(String details, String key, String iv) async {
    final String version = await _methodChannel
        .invokeMethod('decodeData', {'details': details, 'key': key, 'iv': iv});

    return version;
  }*/

  /*let isSingleScan : String = dict["isSingleScan"] ?? ""
  let isPlaySound : String = dict["isPlaySound"] ?? ""
  let isOnFlase : String = dict["isOnFlase"] ?? ""*/

  Future<String> scanBarCodes(
      String isSingleScan,
      String isPlaySound,
      String isOnFlase,
      String themeColor,
      String isGoogleVision,
      String nextBtn,
      String scanText) async {
    final String version = await _methodChannel.invokeMethod('scanBarCodes', {
      'isGoogleVision': isGoogleVision,
      'isSingleScan': isSingleScan,
      'isPlaySound': isPlaySound,
      'isOnFlase': isOnFlase,
      'themeColor': themeColor,
      'nextBtn': nextBtn,
      'scanText': scanText
    });

    return version;
  }

  Future<String> getUniqueNumber() async {
    final String version = await _methodChannel.invokeMethod('UniqueNumber');

    return version;
  }

  Future<String> checkInternet() async {
    final String version = await _methodChannel.invokeMethod('checkInternet');

    return version;
  }

  Future<String> restorePreferences() async {
    final String version =
        await _methodChannel.invokeMethod('restorePreferences');

    return version;
  }

  Future<String> openDateSettings() async {
    final String version =
        await _methodChannel.invokeMethod('openDateSettings');

    return version;
  }

  Future<String> checkBarCode(String code) async {
    final String version =
        await _methodChannel.invokeMethod('checkBarCode', {'code': code});

    return version;
  }

  Future<String> pickFileFromGallary(
      String plateform, String numberOfFiles) async {
    if (plateform.toLowerCase() == "IOS".toLowerCase()) {
      return "not allowd";
    }

    final String version = await _methodChannel
        .invokeMethod('pickFileFromGallary', {'numberOfFiles': numberOfFiles});

    return version;
  }

  Future<String> checkSqliteChiper(
      String androidDBPath, String flutterDBPath) async {
    final String version = await _methodChannel.invokeMethod(
        'checkSqliteChiper',
        {'androidDBPath': androidDBPath, 'flutterDBPath': flutterDBPath});

    return version;
  }

  /*Future<String> stopDatabaseLog(String path) async {
    final String version =
        await _methodChannel.invokeMethod('stopDatabaseLog', {'path': path});

    return version;
  }*/

  Future<String> checkPermission(String plateform) async {
    if (plateform.toLowerCase() == "IOS".toLowerCase()) {
      return "not allowd";
    }

    final String version = await _methodChannel.invokeMethod('checkPermission');
    return version;
  }

  Future<String> getTimeFromUTC(String date, String format) async {
    final String version = await _methodChannel
        .invokeMethod('getTimeFromUTC', {'date': date, 'format': format});

    return version;
  }

  Stream<Map> get onBatteryStateChanged {
    if (_onBatteryStateChanged == null) {
      _onBatteryStateChanged = _eventChannel
          .receiveBroadcastStream()
          .map((dynamic event) => _parseBatteryState(event));
    }
    return _onBatteryStateChanged;
  }

  Map _parseBatteryState(Map state) {
    print("_parseBatteryState : $state");

    /*switch (state) {
      case 'full':
        return BatteryState.full;
      case 'charging':
        return BatteryState.charging;
      case 'discharging':
        return BatteryState.discharging;
      default:
        throw ArgumentError('$state is not a valid BatteryState.');
    }*/

    //Map valueMap = json.decode(state);

    return state;
  }

//basf hk perameters
/*parameterPost.put("FileArray", encodeDb);
  parameterPost.put("FileName", strFilename);
  parameterPost.put("DeviceId", appPrefs.getDeviceID());
  parameterPost.put("PersonId", String.valueOf(appPrefs.getintGlCode()));  == fk_EmpGlCode
  parameterPost.put("Version", getString(R.string.version_value));
  parameterPost.put("SubModuleName", Constants.SUB_MODULE); = ClientName
  parameterPost.put(Constants.PARAM_API_TOKEN, appPrefs.getVarAPIToken()); == UserName*/

}
